library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.std_logic_unsigned.all;

entity max10_adc_lcd is
  port(
	ireset_n  : in  std_logic;
	iclk      : in  std_logic;
	oled_out  : out std_logic_vector(4 downto 0);
	olcd_data : inout std_logic_vector(3 downto 0);
	olcd_rs   : out std_logic;
	olcd_e    : out std_logic
  );
end max10_adc_lcd;

architecture RTL_max10_adc_lcd of max10_adc_lcd is
component pll
	PORT
	(
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0		: OUT STD_LOGIC ;
		c1		: OUT STD_LOGIC ;
		locked		: OUT STD_LOGIC 
	);
end component;

component m10_adc_lcd is
	port (
			adc_clk_clk       : in  std_logic                    := 'X'; -- clk
			adc_locked_export : in  std_logic                    := 'X'; -- export
			clk_clk           : in  std_logic                    := 'X'; -- clk
			lcd_ctrl_export   : out std_logic_vector(1 downto 0);        -- export
			lcd_data_export   : out std_logic_vector(3 downto 0);        -- export
			led_export        : out std_logic_vector(4 downto 0);        -- export
			reset_reset_n     : in  std_logic                    := 'X'  -- reset_n
	);
end component;

signal pll_cpu_clk, pll_adc_clk, pll_locked : std_logic;
signal led_out : std_logic_vector(4 downto 0);
signal lcd_data : std_logic_vector(3 downto 0);
signal lcd_ctrl : std_logic_vector(1 downto 0);
begin
upll:pll
	PORT MAP
	(
		areset		=> not ireset_n,
		inclk0		=> iclk,
		c0		=> pll_cpu_clk,
		c1		=> pll_adc_clk,
		locked		=> pll_locked
	);

u0:m10_adc_lcd
	port map (
		adc_clk_clk       => pll_adc_clk,
		adc_locked_export => pll_locked,
		clk_clk           => pll_cpu_clk,
		lcd_ctrl_export   => lcd_ctrl,
		lcd_data_export   => olcd_data,
		led_export        => led_out,
		reset_reset_n     => ireset_n
	);

	olcd_rs  <= lcd_ctrl(1);
	olcd_e   <= lcd_ctrl(0);
	oled_out <= not led_out;

end RTL_max10_adc_lcd;
