#include <stdio.h>
#include <unistd.h>
#include <system.h>
#include "altera_avalon_pio_regs.h"
#include "altera_modular_adc.h"

#define AIN_CONV(_x) (( 3300*(_x)) >> 11)

void lcd_ctrl_write8(unsigned char data)
{
// bit 1: lcd_rs
// bit 0: lcd_e
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x01); // RS=0, E=1
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_DATA_PIO_BASE, data); // Write Data Assert
	usleep(1); // delay > 140ns
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x00); // RS=0, E=0
	usleep(1); // delay > 1000ns
}

void lcd_ctrl_write4(unsigned char data)
{
// bit 1: lcd_rs
// bit 0: lcd_e
// Upper 4 bit
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x01); // RS=0, E=1
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_DATA_PIO_BASE, data >> 4); // Write Data Assert
	usleep(1); // delay > 140ns
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x00); // RS=0, E=0
	usleep(1); // delay > 1000ns
// Lower 4 bit
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x01); // RS=0, E=1
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_DATA_PIO_BASE, data); // Write Data Assert
	usleep(1); // delay > 140ns
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x00); // RS=0, E=0
	usleep(1); // delay > 1000ns
}

void lcd_char_write(unsigned char data)
{
// bit 1: lcd_rs
// bit 0: lcd_e
// Upper 4 bit
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x03); // RS=1, E=1
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_DATA_PIO_BASE, data >> 4); // Write Data Assert
	usleep(20); // delay > 140ns
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x02); // RS=0, E=0
	usleep(20); // delay > 1000ns
// Lower 4 bit
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x03); // RS=0, E=1
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_DATA_PIO_BASE, data); // Write Data Assert
	usleep(20); // delay > 140ns
	IOWR_ALTERA_AVALON_PIO_DATA(LCD_CTRL_PIO_BASE, 0x02); // RS=0, E=0
	usleep(20); // delay > 1000ns
}

void lcd_str_write(unsigned char *data)
{
	while(*data) {
		lcd_char_write(*data++);
	}
}

void lcd_clear()
{
	lcd_ctrl_write4(0x01);
}

void lcd_set_cursor(unsigned char x, unsigned char y)
{
	if((x >= 0 && x <= 15) || (y >= 0 && y <= 1)) {
		lcd_ctrl_write4(0x80 + y * 0x40 + x);
	}
}

void lcd_init()
{
	usleep(40000);
//	lcd_ctrl_write8(0x03); // Function Set:8-bit, 1-line
//	usleep(40);
	lcd_ctrl_write8(0x03); // Function Set:8-bit, 1-line
	usleep(40);
	lcd_ctrl_write8(0x03); // Function Set:8-bit, 1-line
	usleep(40);
	lcd_ctrl_write8(0x02); // Function Set:4-bit, 1-line
	usleep(40);
	lcd_ctrl_write4(0x28); // Function Set:4-bit, 2-line
	usleep(40);
	lcd_ctrl_write4(0x0c); // Display ON,  Cursor OFF, Cursor Blink OFF
	usleep(40);
	lcd_ctrl_write4(0x01); // Clear Display
	usleep(1600);
	lcd_ctrl_write4(0x06); // Entry Mode Set: left->right, non-shift
	usleep(40);
}

int main()
{
	int data;

// lcd controller initialize
	lcd_init();

// 1st column message output
	lcd_str_write("Button Status");
	lcd_set_cursor(0, 1);

// adc initialize
	adc_set_mode_run_once(MODULAR_ADC_0_SEQUENCER_CSR_BASE);

// endless loop
	for(;;) {
// adc start
		adc_start(MODULAR_ADC_0_SEQUENCER_CSR_BASE);

// wait for sequencer end
		adc_wait_for_interrupt(MODULAR_ADC_0_SAMPLE_STORE_CSR_BASE);
		adc_clear_interrupt_status(MODULAR_ADC_0_SAMPLE_STORE_CSR_BASE);

// button status sampling
		data = IORD(MODULAR_ADC_0_SAMPLE_STORE_CSR_BASE, 0);

		if(data < 0x100) {
			lcd_set_cursor(0, 1); lcd_str_write("Right ");
		} else if(data < 0x180) {
			lcd_set_cursor(0, 1); lcd_str_write("Up    ");
		} else if(data < 0x220) {
			lcd_set_cursor(0, 1); lcd_str_write("Down  ");
		} else if(data < 0x290) {
			lcd_set_cursor(0, 1); lcd_str_write("Left  ");
		} else if(data < 0x2c0) {
			lcd_set_cursor(0, 1); lcd_str_write("Select");
		} else {
			lcd_set_cursor(0, 1); lcd_str_write("None  ");
		}

		usleep(10000);
	}
	return 0;
}
